// (c) Thomas Kamps, 2009
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the7 implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include <QMessageBox>
#include <QTableWidget>
#include <QSettings>
#include <QCloseEvent>
#include <QDateTime>
#include "capitalism.h"
#include "ui_capitalism.h"
#include "src/Monopd/player.h"

Capitalism::Capitalism(QTcpSocket* sok, QWidget *parent)
                       :QMainWindow(parent), ui(new Ui::Capitalism) {
	ui->setupUi(this);

	setClient(new MonopdClient(sok, this));

	ui->dockEstate->hide();
	QSettings s;
	resize(s.value("window/size", QVariant(size())).toSize());
	restoreState(s.value("window/layout", "").toByteArray(), 1);
	locked = s.value("window/locked", "false").toBool();
	setLocked();

	//Set Table with games as central widget
	newGames();
	newGameConfig();
	newField();
	setGame(0);

	layout = new QStackedLayout();
	layout->setObjectName("Stack");
	layout->addWidget(games);
	layout->addWidget(gameConfig);
	layout->addWidget(field);
	layout->addWidget(client);
	ui->center->addLayout(layout);
	layout->setCurrentIndex(3);

	eventVisible = false;
	chatColor = false;
	eventColor = false;

	selectedField = -1;

	config = new Config(this);

	//Config
	connect(config, SIGNAL(saved(QString, QString)), this, SLOT(setData(QString, QString)));
	connect(config, SIGNAL(saved(QString, bool)), this, SLOT(setData(QString, bool)));

	//Menu
	connect(ui->acLeaveGame, SIGNAL(triggered()), this, SLOT(leaveGame()));
	connect(ui->acDisconnect, SIGNAL(triggered()), this, SLOT(disconnectFromServer()));
	connect(ui->acExitCapitalsm, SIGNAL(triggered()), this, SLOT(exit()));
	connect(ui->acConfig, SIGNAL(triggered()), config, SLOT(show()));

	//Turn
	ui->bRoll->cmd = ".r";
	ui->bBuy->cmd = ".eb";
	ui->bAuction->cmd = ".ea";
	ui->bEndturn->cmd = ".E";
	connect(ui->bRoll, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bBuy, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bBuy, SIGNAL(clicked()), field, SLOT(clearButtons()));
	connect(ui->bAuction, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bAuction, SIGNAL(clicked()), field, SLOT(clearButtons()));
	connect(ui->bEndturn, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bEndturn, SIGNAL(clicked()), field, SLOT(clearButtons()));

	//Jail
	ui->bJailRoll->cmd = ".jr";
	ui->bCard->cmd = ".jc";
	ui->bPay->cmd = ".jp";
	connect(ui->bJailRoll, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bJailRoll, SIGNAL(clicked()), field, SLOT(clearButtons()));
	connect(ui->bCard, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bCard, SIGNAL(clicked()), field, SLOT(clearButtons()));
	connect(ui->bPay, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bPay, SIGNAL(clicked()), field, SLOT(clearButtons()));

	//Estate
	connect(ui->bSell, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bBuyHouse, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bSellHouse, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bMortage, SIGNAL(cmdClicked(QString)), client, SLOT(send(QString)));
	connect(ui->bBuyHouseRow, SIGNAL(cmdClicked(QString)), this, SLOT(buyRow()));
	connect(ui->bSellHouseRow, SIGNAL(cmdClicked(QString)), this, SLOT(sellRow()));

	//chat
	connect(ui->bSend, SIGNAL(clicked()), this, SLOT(sendMsg()));
	connect(ui->tfMess, SIGNAL(returnPressed()), this, SLOT(sendMsg()));

	//news
	connect(ui->dockChat, SIGNAL(visibilityChanged(bool)), this, SLOT(normalChatTitle(bool)));
	connect(ui->dockEvents, SIGNAL(visibilityChanged(bool)), this, SLOT(normalEventTitle(bool)));

	if (s.value("connection/server", QVariant("")) != "") {
		if(s.value("connection/lastMessage", QVariant(0)).toUInt()+180 > QDateTime::currentDateTime().toTime_t()) {
		if (QMessageBox::question(this, tr("Reconnect"), tr("Capitalism seems to be chrashed recently. Would you try to reconnect to the last game?"),
									  QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes) {
				client->reconnect();
		}
		}
	}
}

Capitalism::~Capitalism() {
	delete ui;
}

MonopdGame* Capitalism::getGame() {
	return game;
}

void Capitalism::setGame(MonopdGame* g) {
	game = g;
	if (field == 0) return;
	field->setGame(g);
	field->setClientId(client->getClientPlayerId());
	foreach(Trade* t, trades) {
		t->setGame(g);
	}
}

void Capitalism::roll() {
	if (client == 0) return;
	client->send(".r");
}

void Capitalism::buy() {
	if (client == 0) return;
	client->send(".eb");
}

void Capitalism::jailRoll() {
	if (client == 0) return;
	client->send(".jr");
}

void Capitalism::jailCard() {
	if (client == 0) return;
	client->send(".jc");
}

void Capitalism::jailPay() {
	if (client == 0) return;
	client->send(".jp");
}

void Capitalism::buyRow(int gr) {
	MonopGroup* g = game->getGroup(gr);

	//Check if player can pay this
	int neededMoney = 0;
	for (int i = 0; i < g->getNumEstates(); i++) {
		neededMoney += g->getEstate(i)->getHousePrice();
	}
	Player* pl = game->getPlayer(client->getClientPlayerId());
	if (neededMoney > pl->getMoney()) {
		QMessageBox::information(this, "Buying Houserow", tr("Cannot buy houserow, because you don't have enough money. You need %1").arg(QString::number(neededMoney)));
		return;
	}

	//Buy them, if possible
	for (int i = 0; i < g->getNumEstates(); i++) {
		send(".hb"+QString::number(g->getEstate(i)->getId()));
	}
}

void Capitalism::sellRow(int gr) {
	MonopGroup* g = game->getGroup(gr);
	for (int i = 0; i < g->getNumEstates(); i++) {
		send(".hs"+QString::number(g->getEstate(i)->getId()));
	}
}

void Capitalism::buyRow() {
	buyRow(game->getField(selectedField)->getGroup());
}

void Capitalism::sellRow() {
	sellRow(game->getField(selectedField)->getGroup());
}

void Capitalism::sendMsg() {
	if (client == 0) return;
	client->send(ui->tfMess->text());
	ui->tfMess->setText("");
}

void Capitalism::setSelectedField(MonopdField* f) {
	if ((f == 0) || (f->getGroup() == -1)) {
		selectedField = -1;
	} else {
		selectedField = f->getId();
	}
	updateEstateDock();
}

void Capitalism::updateEstateDock() {
	if (selectedField == -1) {
		ui->dockEstate->setWindowTitle(tr("Estate"));
		ui->dockEstate->setEnabled(false);
	} else {
		if (game == 0) return;
		MonopdField* f = game->getField(selectedField);
		ui->dockEstate->setWindowTitle(tr("Estate: %1").arg(f->getName()));
		ui->dockEstate->setEnabled(f->getOwner() == client->getClientPlayerId());

		ui->bMortage->setEnabled(f->getCanToggleMortage());
		if (f->getMortaged()) ui->bMortage->setIcon(QIcon(":/images/tools/unmortage.png"));
		else ui->bMortage->setIcon(QIcon(":/images/tools/mortage.png"));
		ui->bMortage->cmd = ".em"+QString::number(selectedField);

		ui->bBuyHouse->setEnabled(f->getCanBuyHouse());
		ui->bBuyHouse->cmd = ".hb"+QString::number(selectedField);

		ui->bSellHouse->setEnabled(f->getCanSellHouse());
		ui->bSellHouse->cmd = ".hs"+QString::number(selectedField);

		ui->bSell->cmd = ".es"+QString::number(selectedField);

		MonopGroup* g = game->getGroup(f->getGroup());
		bool canBuyRow = g->canBuyHouseRow();
		bool canSellRow = g->canSellHouseRow();
		ui->bBuyHouseRow->setEnabled(canBuyRow);
		ui->bSellHouseRow->setEnabled(canSellRow);
	}
}

void Capitalism::normalChatTitle(bool vis) {
	chatVisible = vis;
	if (!vis) return;
	ui->dockChat->setWindowTitle(tr("Chat"));
}

void Capitalism::normalEventTitle(bool vis) {
	eventVisible = vis;
	if (!vis) return;
	ui->dockEvents->setWindowTitle(tr("Messages"));
}

void Capitalism::setClient(MonopdClient* c) {
	client = c;
	connect(client, SIGNAL(connected()), this, SLOT(setCurGame()));

	connect(client, SIGNAL(fieldUpdate()), this, SLOT(drawField()));
	connect(client, SIGNAL(fieldUpdate()), this, SLOT(updateEstateDock()));
	connect(client, SIGNAL(playerAdded(Player*)), this, SLOT(addToList(Player*)));
	connect(client, SIGNAL(chatMessage(int, QString)), this, SLOT(showChat(int, QString)));
	connect(client, SIGNAL(serverMessage(QString, QString)), this, SLOT(showEvent(QString, QString)));
	connect(client, SIGNAL(playerUpdated()), this, SLOT(playerUpdate()));
	connect(client, SIGNAL(newGameList()), this, SLOT(updateGameList()));
	connect(client, SIGNAL(curGameChanged()), this, SLOT(setCurGame()));
	connect(client, SIGNAL(statusChanged(QString)), this, SLOT(setCurGame()));
	connect(client, SIGNAL(gameMasterChanged(int,int)), this, SLOT(setOptionsEnabled(int,int)));

	connect(client, SIGNAL(tradeUpdate(QString,int,int,int)), this, SLOT(tradeUpdate(QString,int,int,int)));
	connect(client, SIGNAL(tradePlayerUpdate(int,int,bool)), this, SLOT(tradePlayerUpdate(int,int,bool)));
	connect(client, SIGNAL(tradeEstateUpdate(int,int,int)), this, SLOT(tradeEstateUpdate(int,int,int)));
	connect(client, SIGNAL(tradeCardUpdate(int,int,int)), this, SLOT(tradeCardUpdate(int,int,int)));
	connect(client, SIGNAL(tradeMoneyUpdate(int,int,int,int)), this, SLOT(tradeMoneyUpdate(int,int,int,int)));
}

void Capitalism::playerUpdate() {
	Player* p = client->getPlayer(client->getClientPlayerId(), false);
	if (p == 0) return;
	ui->bRoll->setEnabled((client->getCurGameId() != -1) && p->getHasTurn() && p->getCanRoll());
	ui->bAuction->setEnabled((client->getCurGameId() != -1) && p->getHasTurn() && p->getCanAuction());
	ui->bBuy->setEnabled((client->getCurGameId() != -1) && p->getHasTurn() && p->getCanBuyEstate());
	ui->dockJail->setEnabled(p->getInJail() && p->getHasTurn());
	ui->bCard->setEnabled(p->getCanUseCard());
	ui->bEndturn->setEnabled(p->getHasTurn() && !p->getInJail() && p->getCanBuyEstate() && !p->getCanAuction());
}

void Capitalism::updateGameList() {
	if (games == 0) return;
	if (client == 0) return;
	if (client->getCurGameId() != -1) return;
	games->clearContents();
	games->setColumnCount(6);
	games->setHorizontalHeaderLabels(QStringList(tr("Name")) << "" <<tr("Description") << tr("Players") << "joinCmd");
	games->setVerticalHeaderLabels(QStringList());

	games->setRowCount(0);
	int r = 0;
	for (int i = 0; i < client->getNumGames(); i++) {
		MonopdGame* g = client->getGameFromList(i);
		if (!g->getListed()) continue;
		games->insertRow(r);

		Player* p = client->getPlayer(g->getMaster(), false);
		QString name = tr("Unknown");
		if (p != 0) name = p->getName();
		QTableWidgetItem* it = new QTableWidgetItem(tr("Join %1 from %2").arg(g->getName()).arg(name));
		bool cbj = g->getCanBeJoined();
		if (cbj) it->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		else it->setFlags(Qt::ItemIsEnabled);
		games->setItem(r, 0, it);

		it = new QTableWidgetItem(tr("Spectate"));
		it->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		games->setItem(r, 1, it);

		it = new QTableWidgetItem(g->getDesc());
		it->setFlags(Qt::ItemIsEnabled);
		games->setItem(r, 2, it);

		it = new QTableWidgetItem(QString::number(g->getNumPlayer()));
		it->setFlags(Qt::ItemIsEnabled);
		games->setItem(r, 3, it);

		it = new QTableWidgetItem(".gj"+QString::number(g->getGameId()));
		it->setFlags(Qt::ItemIsEnabled);
		games->setItem(r, 4, it);

		it = new QTableWidgetItem(".gS"+QString::number(g->getGameId()));
		it->setFlags(Qt::ItemIsEnabled);
		games->setItem(r, 5, it);

		r++;
	}

	for (int i = 0; i < client->getNumTemplates(); i++) {
		games->insertRow(r);

		QTableWidgetItem* it = new QTableWidgetItem(tr("Start new %1").arg(client->getTemplateFromList(i)->getName()));
		it->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		it->setBackground(QBrush(QColor(192,255,192)));
		games->setItem(r, 0, it);

		it = new QTableWidgetItem(client->getTemplateFromList(i)->getDesc());
		it->setFlags(Qt::ItemIsEnabled);
		games->setItem(r, 2, it);

		it = new QTableWidgetItem("");
		it->setFlags(Qt::ItemIsEnabled);
		games->setItem(r, 3, it);

		it = new QTableWidgetItem(".gn"+client->getTemplateFromList(i)->getType());
		it->setFlags(Qt::ItemIsEnabled);
		games->setItem(r, 4, it);

		r++;
	}
	games->hideColumn(1); //Hide specation option, until there is a working specating mode by Monopd
	games->hideColumn(4);
	games->hideColumn(5);
	games->resizeColumnsToContents();
}

void Capitalism::showChat(int pid, QString msg) {
	Player* p;
	if (client->getCurGameId() != -1) {
		p = game->getPlayer(pid);
	}
	QString bg = "#ffffff";
	if (chatColor) bg = "#f0f0f0";
	chatColor = !chatColor;

	QString name = tr("Unknown Player");
	QString pColor = "#808080";

	p = client->getPlayer(pid, false);
	if (p != 0) {
		name = p->getName();
		pColor = p->getChatColor();
	}

	chatHtml = chatHtml+"<p style=\"margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; background:"+bg+"\"><span style=\"color:"+pColor+"\">"+name+":</span> "+msg+"</p>";
	ui->chat->setHtml(chatHtml+"<a name=\"end\" style=\"font-size:1px\">&nbsp;</a>");
	ui->chat->scrollToAnchor("end");

	if (!chatVisible) {
		ui->dockChat->setWindowTitle("*Chat*");
	}
}

void Capitalism::showEvent(QString type, QString msg) {
	//info, error, standby, and startgame
	QString color = "#000000";
	if (type == "info") color="#000000";
	if (type == "error") color="#800000";
	if (type == "standby") color="#008000";
	if (type == "startgame") color="#000080";

	QString bg = "#ffffff";
	if (eventColor) bg="#f0f0f0";

	eventHtml = eventHtml+"<p style=\"margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; background:"+bg+"\"><span style=\"color:"+color+"\">"+msg+"</span></p>";
	ui->events->setHtml(eventHtml+"<a name=\"end\" style=\"font-size:1px\">&nbsp;</a>");
	ui->events->scrollToAnchor("end");

	eventColor = !eventColor;

	if (!eventVisible) {
		ui->dockEvents->setWindowTitle(tr("*Messages*"));
	}
}

void Capitalism::setCurGame() {
	int gid = client->getCurGameId();
	if (gid == -1) {
		setGame(0);
		setSelectedField(0);
		ui->bRoll->setEnabled(false);
		ui->bBuy->setEnabled(false);
		ui->bAuction->setEnabled(false);
		ui->bEndturn->setEnabled(false);
		ui->dockJail->setEnabled(false);
		layout->setCurrentIndex(0);
	} else {
		setGame(client->getGame(gid));
		//Set gameconfig as central widget
		if (game->getStatus() == "config") {
			layout->setCurrentIndex(1);
			gameConfig->setDesc(game->getDesc());
			gameConfig->setOptionsEnabled(game->getMaster() == client->getClientPlayerId());
		} else if (game->getStatus() == "run") {
			gameConfig->removeAllOptions();
			layout->setCurrentIndex(2);
		}
	}
}

void Capitalism::setOptionsEnabled(int gid, int newMaster) {
	if (gid == -1) return;
	if (client->getCurGameId() == -1) return;
	if (gid == client->getCurGameId()) gameConfig->setOptionsEnabled(newMaster == client->getClientPlayerId());
}

void Capitalism::startGame(int r, int c) {
	if (games == 0) return;
	QTableWidgetItem* it = 0;
	if (c == 0)
		it = games->item(r, 4);
	else if (c == 1)
		it = games->item(r, 5);

	if (it != 0) send(it->text());
}

void Capitalism::addToList(Player* p) {
	if (p->getId() == client->getClientPlayerId()) {
		ui->playerInfos->insertWidget(0, p->playerWidget);
	} else {
		ui->playerInfos->insertWidget(ui->playerInfos->indexOf(ui->anchor), p->playerWidget);
	}
	ui->playerInfos->update();
	field->addPlayerAuction(p);

	connect(p->playerWidget, SIGNAL(command(QString)), client, SLOT(send(QString)));
}

void Capitalism::newGames() {
	games = new QTableWidget(this);
	games->setObjectName("Lobby");
	games->setHorizontalHeaderLabels(QStringList(tr("Name")) << tr("Description") << tr("Players"));

	connect(games, SIGNAL(cellDoubleClicked(int,int)), this, SLOT(startGame(int, int)));
}

void Capitalism::newGameConfig() {
	gameConfig = new GameConfig(this);
	gameConfig->setObjectName(tr("Config"));
	if (client == 0) return;

	connect(client, SIGNAL(configUpdate(QString,QString,QString, QString)), gameConfig, SLOT(configUpdate(QString, QString, QString, QString)));

	connect(gameConfig, SIGNAL(optionChanged(QString)), client, SLOT(send(QString)));
}

void Capitalism::newField() {
	field = new Field(this);
	field->setMinimumWidth(500);
	field->setObjectName("Field");

	if (client == 0) return;
	connect(client, SIGNAL(gotField(int)), field, SLOT(showField(int)));
	connect(client, SIGNAL(gotText(QString)), field, SLOT(appendLine(QString)));
	connect(client, SIGNAL(clearedText()), field, SLOT(clearText()));
	connect(field, SIGNAL(buttonClicked(QString)), client, SLOT(send(QString)));
	connect(client, SIGNAL(gotButton(QString,QString)), field, SLOT(addButton(QString,QString)));
	connect(client, SIGNAL(playerAdded(Player*)), field, SLOT(addPlayerAuction(Player*)));

	connect(client, SIGNAL(auctionStarted()), field, SLOT(showAuction()));
	connect(client, SIGNAL(auctionIdUpdate(int)), field, SLOT(updatAuctionId(int)));
	connect(client, SIGNAL(auctionFieldUpdate(int)), field, SLOT(updateAuctionField(int)));
	connect(client, SIGNAL(auctionBidUpdate(int,int)), field, SLOT(bidUpdate(int,int)));
	connect(client, SIGNAL(auctionStatusUpdate(int)), field, SLOT(updateAuctionStatus(int)));

	connect(field->auction, SIGNAL(bid(QString)), client, SLOT(send(QString)));

	connect(field, SIGNAL(fieldSelected(MonopdField*)), this, SLOT(setSelectedField(MonopdField*)));
	connect(field, SIGNAL(rowBuyed(int)), this, SLOT(buyRow(int)));
	connect(field, SIGNAL(rowSelled(int)), this, SLOT(sellRow(int)));
}

Trade* Capitalism::getTrade(int tid) {
	Trade* t = 0;
	for (int i = 0; i < trades.size(); i++)
		if (trades.value(i)->getId() == tid) t = trades.value(i);
	return t;
}

void Capitalism::removeTrade(Trade* t) {
	trades.removeAll(t);
	delete t;
}

void Capitalism::setLocked() {
	QList<QDockWidget*> docks;
	docks << ui->dockChat << ui->dockEvents << ui->dockPlayers << ui->dockTurn << ui->dockJail << ui->dockEstate;
	foreach(QDockWidget* d, docks) {
		if (!locked) {
			d->setFeatures(QDockWidget::AllDockWidgetFeatures);
		} else {
			d->setFeatures(QDockWidget::NoDockWidgetFeatures);
		}
	}
}

void Capitalism::send(QString msg) {
	if (client == 0) return;
	client->send(msg);
}

void Capitalism::drawField() {
	if (field == 0) return;
	if (client->getCurGameId() != -1) {
		MonopdGame* g = client->getGame(client->getCurGameId());
		if (g->getStatus() != "config")
			field->update();
	}
}

void Capitalism::tradeUpdate(QString type, int tradeId, int actor, int rev) {
	if (type == "new") {
		Trade* t = new Trade(tradeId);
		t->setRev(0);
		t->setGame(game);
		if (actor != -1) t->updateTradeActor(actor);
		trades.append(t);
		connect(t, SIGNAL(tradeCommand(QString)), client, SLOT(send(QString)));
		t->show();
	} else if (type == "edit") {
		Trade* t = getTrade(tradeId);
		if (actor != -1) t->updateTradeActor(actor);
		if (t == 0) return;
		if (rev > 0) t->setRev(rev);
	} else if ((type == "rejected") || (type == "completed")) {
		Trade* t = getTrade(tradeId);
		if (t == 0) return;
		removeTrade(t);
	}
}

void Capitalism::tradePlayerUpdate(int tradeId, int playerid, bool accept) {
	Trade* t = getTrade(tradeId);
	if (t == 0) return;

	t->updateTradeAccept(playerid, accept);
}

void Capitalism::tradeEstateUpdate(int tradeId, int estateId, int targetPlayerId) {
	Trade* t = getTrade(tradeId);
	if (t == 0) return;

	t->updateEstateTrade(estateId, targetPlayerId);
}

void Capitalism::tradeCardUpdate(int tradeId, int cardId, int targetPlayerId) {
	Trade* t = getTrade(tradeId);
	if (t == 0) return;

	t->updateCardTrade(cardId, targetPlayerId);
}

void Capitalism::tradeMoneyUpdate(int tradeId, int fromPlayerId, int money, int targetPlayerId) {
	Trade* t = getTrade(tradeId);
	if (t == 0) return;

	t->updateMoneyTrade(fromPlayerId, money, targetPlayerId);
}

void Capitalism::setData(QString type, QString data) {
	if (type == "name") send(".n"+data);
	if (type == "image") send(".pi"+data);
}

void Capitalism::setData(QString type, bool data) {
	if (type == "locked") {
		locked = data;
		setLocked();
	}
}

void Capitalism::leaveGame() {
	while (trades.size() > 0) {
		Trade* t = trades.value(0);
		t->reject();
		removeTrade(t);
	}
	client->leaveGame();
	setGame(0);
	setCurGame();
}

void Capitalism::disconnectFromServer() {
	leaveGame();
	games->setRowCount(0);
	client->disconnectFromHost();
	client->updateServerList();
	layout->setCurrentIndex(3);
}

void Capitalism::exit() {
	close();
}

void Capitalism::closeEvent(QCloseEvent *event) {
	bool close = true;
	if (game != 0) {
		close = false;
		if (QMessageBox::question(this, tr("Close current game?"), tr("Do you really want to leave the current game?"),
								  QMessageBox::Yes | QMessageBox::No) == QMessageBox::Yes) {
			close = true;
		}
	}

	if (close) {
		leaveGame();
		client->disconnectFromHost();

		QSettings s;
		s.setValue("window/size", QVariant(size()));
		s.setValue("window/layout", QVariant(saveState(1)));
		s.setValue("window/locked", QVariant(locked));
		event->accept();
	} else {
		event->ignore();
	}
}
